
# q-expansion of newform 882.6.a.i, downloaded from the LMFDB on 10 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 882
weight = 6
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-4], [0], [24], [0], [-66], [-98], [-216], [340], [1038], [2490], [7048], [-12238], [6468], [-15412], [20604], [-32490], [34224], [-35654], [12680], [42642], [-33734], [-85108], [-106764], [34884], [-18662], [153084], [-35864], [95454], [212222], [-62106], [-53044], [69324], [-129846], [104356], [-217194], [221000], [378370], [104816], [-426972], [331068], [400194], [-588098], [-939342], [338390], [237942], [-204464], [-348724], [-1470056], [-589560], [1045342], [-651222], [513462], [694714], [-1396080], [-1005204], [-1253010], [-1760688], [-770528], [707738], [-2304318], [-1609028], [517020], [-1350020], [1345380], [-256154], [-1846290], [-3332380], [-1634806], [841530], [977242], [3458568], [3473010], [-3119936], [-2016730], [-5380828], [807432], [-891390], [-1123454], [-1720374], [-77246], [-5206152], [1718474], [580626], [-4150874], [-3884072], [2314986], [1922814], [6862154], [2971668], [4874228], [-8173008], [2343972], [316928], [5200410], [-4867732], [426888], [-9416208], [1840392], [979108], [5961170], [8730248], [3010662], [11757264], [-13157814], [-10334440], [7881334], [-554568], [-9203688], [-8542950], [9615550], [-2212640], [-7962154], [13739682], [8701132], [445412], [8001186], [15840244], [1301868], [-7341486], [6189342], [19668970], [7182590], [-18919212], [-21220398], [-16327580], [5404698], [22119494], [25581888], [9294376], [-3406994], [21813536], [-3792462], [-20148256], [11842742], [2977908], [20244070], [-7379532], [-13628924], [-14954772], [-28724202], [15226540], [33100062], [-13591540], [-3139362], [-12708074], [-29831172], [19221538], [-26565468], [9168436], [29219598], [9712862], [16537248], [-23948692], [-4628460], [20612588], [3467490], [-36122704], [12936612], [-50139362], [-10556832], [3146838], [-52282890], [-24823468], [13307724], [-8177046], [-13246512], [-14854984], [3337690], [-28787218], [-40667988], [30669846], [-57889790], [-7566774], [-33152746], [24463144], [31374672], [196016], [51080418], [-53921504], [-3102494], [44171636], [2565948], [24041978], [22613832], [63414894], [-35473164], [27017462], [-34942244], [3783302], [10750104], [88285642], [-74982882], [-7512892], [50423124], [50436558], [81410064], [-43457698], [-54169726], [55525764], [54441180], [-32169714], [4451944], [56284246], [67783162], [-68660172], [43817292], [88225288], [-44337354], [-56993610], [-48811148], [-101369662], [-8057280], [59798048], [102542268], [-26837292], [-11369858], [100083896], [54185664], [56551518], [-13559478], [18267254], [-35080136], [77237526], [-27506320], [-58481220], [-61748734], [114626676], [100402614], [-71336408], [18772098], [42982334], [9237724], [-47072296], [-115301994], [115318084], [-85091712], [-105455654], [161207118], [130929762], [5118120], [47155434], [-27026660], [37125136], [112305170], [96311796], [138977940], [-92818712], [-113693880], [-7096744], [-120050226], [106917974], [123169896], [-118212342], [146436682], [151610124], [-56544066], [23311802], [135965668], [17421012], [-198501698], [-73672708], [-183063414], [10411246], [155546314], [-88827024], [60942676], [-47338650], [233766120], [97103456], [-143807982], [-3842198], [-148958044], [116057290], [168698024], [131346082], [251568664], [-247183242], [41028242], [171634286], [187756824], [136026024], [-41051980], [136578288], [288611050], [-22449500], [-291893166], [-47044078], [136351038], [-154502888], [95469948], [15059454], [129464028], [103552626], [118421016], [214599758], [228638796], [-155891480], [-204623664], [144502704], [-234530996], [-173461130], [210175002], [-15337564], [-21842466], [-80861608], [-283300918], [350825010], [-264028058], [-163559310], [-188486566], [54689736], [143778618], [364074090], [88611596], [-130052610], [-137545094], [-107503620], [-181737258], [56418754], [402710754], [-8915180], [-221741578], [370553652], [-22327576], [29987934], [-233498702], [-194139304], [64366348], [-155481366], [-429705054], [-363818506], [-306649374], [-385720160], [41557944], [-245003056], [-279199872], [358119470], [-408795636], [-81617102], [222753472], [-14281978], [275829090], [-110669664], [-337716832], [213314586], [-202027842], [-118402922], [-44976388], [-497789880], [-313730820], [-10380212], [-299198554], [86924766], [-441926024], [63281510], [288391824], [-444636336], [-177596964], [-517430058], [-217083414], [-52707226], [504401628], [-148295934], [171988854], [-74609228], [464039798], [451500168], [342662708], [-282528034], [-31865718], [-470799164], [-377713446], [-507113490], [419687848], [340365170], [-123562452], [135641574], [-319627946], [141523560], [-312061190], [-484549260], [-165763194], [-544070896], [123026838], [71588188], [-381430944], [469585616], [691621174], [-136550752], [-54044508], [-577918066], [405740856], [586204686], [-341325428], [546394578], [105789554], [261011176], [410699856], [-343318960], [-437159670], [779589514], [84935286], [-308743444], [-678399852], [454608792], [-358815656], [-268521502], [363398334], [-173495204], [240103272], [-426126578], [-238577358], [-576569646], [-250539340], [-319480138], [-52982136], [843925518], [-577186016], [42748272], [-63025932], [106421802], [193908922], [-171267294], [-778082844], [840795046], [774895164], [172048074], [729149202], [-527296988], [-173216364]]
